@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 package axmud;
 
-# Copyright (C) 2011-2019 A S Lewis
+# Copyright (C) 2011-2020 A S Lewis
 #
 # This program is free software: you can redistribute it and/or modify it under the terms of the GNU
 # General Public License as published by the Free Software Foundation, either version 3 of the
@@ -16,7 +16,7 @@ package axmud;
 #
 #
 # axmud.pl
-# Axmud - a Multi-User Dungeon (MUD) client written in Perl5 / Gtk2
+# Axmud - a Multi-User Dungeon (MUD) client written in Perl5 / Gtk3
 #
 # This script is suitable for all users. Visually-impaired users might prefer to run baxmud.pl
 #   instead, as it automatically enables features for those users, such as built-in text-to-speech
@@ -43,15 +43,15 @@ use vars qw(
 );
 
 $SCRIPT = 'Axmud';              # Name used in system messages
-$VERSION = '1.2.041';           # Version number for this client
-$DATE = '31 Jul 2019';
+$VERSION = '1.3.0';             # Version number for this client
+$DATE = '19 Sep 2020';
 $NAME_SHORT = 'axmud';          # Lower-case version of $SCRIPT; same as the package name above
 $NAME_ARTICLE = 'an Axmud';     # Name with an article
 $BASIC_NAME = 'Axbasic';        # Name of Axmud's built-in scripting library
 $BASIC_ARTICLE = 'an Axbasic';  # Name with an article
-$BASIC_VERSION = '1.003';       # Version number for the Axbasic library
+$BASIC_VERSION = '1.004';       # Version number for the Axbasic library
 $AUTHORS = 'A S Lewis';
-$COPYRIGHT = 'Copyright 2011-2019 A S Lewis';
+$COPYRIGHT = 'Copyright 2011-2020 A S Lewis';
 $URL = 'http://axmud.sourceforge.io/';
 $DESCRIP = 'A modern MUD client for MS Windows, Linux and *BSD';
 
@@ -67,7 +67,7 @@ $NAME_FILE = 'axmud';
 @COMPAT_EXT_LIST = ('axm', 'amd');
 
 # Axmud blind mode: if this flag is TRUE, when Axmud starts the first time, it will do so with
-#   settings optimised for users with a visual impairment
+#   settings optimised for users with a visual impairment. Auto-disabled when test mode is enabled
 $BLIND_MODE_FLAG = FALSE;
 # Axmud safe mode: this flag is (briefly) set to TRUE whenever GA::Session->perlCmd tries to execute
 #   some arbitrary Perl code using the Safe module. In that situation, the error-trapping code below
@@ -76,7 +76,7 @@ $SAFE_MODE_FLAG = FALSE;
 
 # Axmud test mode: if this flag is TRUE, when Axmud starts it automatically connects and logs in to
 #   the a world which is assumed to be running on your local system, without first opening the
-#   Connections window
+#   Connections window. Auto-disabled when blind mode is enabled
 $TEST_MODE_FLAG = FALSE;
 # If $TEST_MODE_FLAG is TRUE, the login details to use. Must be in the form
 #   (world_name, host, port, username, password, online_flag)
@@ -189,7 +189,7 @@ use Scalar::Util qw(looks_like_number);
 use Socket qw(AF_INET SOCK_STREAM inet_aton sockaddr_in);
 use Symbol qw(qualify);
 use Storable qw(lock_nstore lock_retrieve);
-use Time::HiRes qw(gettimeofday);
+use Time::HiRes qw(gettimeofday usleep);
 use Time::Piece;
 
 # Net::SSLeay issues can cause inability to install IO::Socket::SSL on some systems. If it's not
@@ -253,6 +253,8 @@ $SIG{__DIE__} = sub {
 
     if ($CLIENT) {
 
+        my $engines = join('|', @{$CLIENT->{constTTSList}});
+
         # Errors generated by GA::Session->perlCmd cause a chain of errors, because the Perl Safe
         #   module uses its own namespace and can't call GA::Client->writePerlWarning
         # Workaround is to use a global flag and to generate our own error message if it's set
@@ -261,8 +263,17 @@ $SIG{__DIE__} = sub {
             # Regex test initiated by GA::Client->regexCheck
             $TEST_REGEX_ERROR = $_[0];
 
-        } else {
-
+        } elsif (
+            # Catch the exceptions generated by GA::Client->ipv4Get
+            ! ($_[0] =~ m/Could not connect to.*Name or service not known/)
+            && ! ($_[0] =~ m/Timed out while waiting for socket to become ready for reading/)
+            && ! ($_[0] =~ m/Could not connect.*Network is unreachable/)
+            && ! ($_[0] =~ m/SSL connection failed for.*SSL wants a read first/)
+            # Catch the exceptions generated by TTS engines not installed on a Linux system
+            #   (on MS Windows, the location of the executable is predictable, so GA::Client code
+            #   checks for it explicitly)
+            && ! ($_[0] =~ m/Command \'($engines)\' not found in/)
+        ) {
             # We don't know which GA::Session caused the Perl error, but we can leave Axmud in a
             #   (more or less) functional state by halting all client loops and session loops; the
             #   user can restart them, when ready, with the ';restart' command
